/*
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package com.sun.ts.tests.portlet.api.javax_portlet.ActionRequest;

import com.sun.javatest.Status;
import com.sun.ts.tests.portlet.common.client.TSPortletInfo;
import com.sun.ts.tests.portlet.common.client.BasePortletUrlClient;
import com.sun.ts.tests.portlet.common.util.ResultWriter;
import com.sun.ts.tests.portlet.common.client.tags.PortletTCKCustomClientTag;
import com.sun.ts.tests.portlet.common.client.tags.PortletURLClientTag;
import com.sun.ts.tests.common.webclient.http.HttpResponse;
import org.apache.commons.httpclient.HttpState;


/**
 * URLClient and SpecURLClient are used as HTTP test clients to test
 * the Portlet API and specification assertions.  Each assertion test
 * is performed by executing a method in the *URLClient class that has
 * the @testName, @assertion_ids, @test_Strategy, and @assertion tags
 * in its javadoc comments.
 * <p>
 * In each test, at least one HTTP request is made to retrieve the
 * portal page that interacts with portlets that participate in the
 * corresponding test.  These test portlets/servlets, bundled in WAR
 * files, are assumed to be already deployed on the portal server
 * before the test is run.  The initial URL of the portal page is
 * obtained, using either declarative or programmatic configuration,
 * by the base class method <code>getPortalURL</code>, and is set for
 * the test using the <code>setRequestProperty</code> method.  See the
 * Technology Compatibility Kit Requirements chapter of the Portlet
 * Specification for details.  Subsequent requests for the test are
 * done using URLs, generated by PortletURL, that are part of the
 * returned portal pages.
 * <p>
 * The test sets the test success criteria using the
 * <code>setCriteriaProperty</code> method to look for either expected
 * or unexpected substrings in the portal page returned to decide
 * whether a test has passed or failed.
 * <p>
 * Finally, the <code>invoke()</code> method makes the HTTP request
 * and validates the output for the configured success criteria.  In
 * case of failure, this method throws a <code>Fault</code> exception
 * that is caught by the framework to report a failure for the test. 
 */
public class URLClient extends BasePortletUrlClient {
    public static void main(String[] args) {
        URLClient theTests = new URLClient();
        Status status = theTests.run(args, System.out, System.err);
        status.exit();
    }

    /**
    * Returns the name of the default portlet app.
    */
    public String getDefaultPortletApp() {
        return "portlet_jp_ActionRequest_web";
    }

    /*
     * @class.setup_props: ts_home;
     */

    /*
     * @testName: GetAttributeNamesTest
     * @assertion_ids: PORTLET:JAVADOC:168;
     * @test_Strategy: In processAction(), sets couple of attributes and makes sure 
     *         getAttributeNames() returns exact same attributes. 
     * @assertion: Returns an Enumeration containing the names of the 
     *             attributes available to this request. This method returns 
     *             an empty Enumeration if the request has no attributes 
     *             available to it.
     */
    public void GetAttributeNamesTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetAttributeNamesTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetAttributeTest
     * @assertion_ids: PORTLET:JAVADOC:166;
     * @test_Strategy: In processAction(), sets one attribute in PortletRequest and make sure we 
     *          can read that attribute back using PortletRequest.getAttribute()
     * @assertion: Returns the value of the named attribute as an Object, or 
     *             null if no attribute of the given name exists. Attribute 
     *             names should follow the same conventions as package names. 
     *             This specification reserves names matching java.*, and 
     *             javax.*. In a distributed portlet web application the 
     *             Object needs to be serializable. 
     */
    public void GetAttributeTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetAttributeTestPortlet");


        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetAttributeInvalidTest
     * @assertion_ids: PORTLET:JAVADOC:166;
     * @test_Strategy: In processAction(), Checks that getAttribute() returns null if no attribute 
     *                of the given name exists.
     * @assertion: Returns the value of the named attribute as an Object, or 
     *             null if no attribute of the given name exists. Attribute 
     *             names should follow the same conventions as package names. 
     *             This specification reserves names matching java.*, and 
     *             javax.*. In a distributed portlet web application the 
     *             Object needs to be serializable. 
     */
    public void GetAttributeInvalidTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetAttributeInvalidTestPortlet");

        
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetAuthTypeWithoutProtectionTest
     * @assertion_ids: PORTLET:JAVADOC:169;PORTLET:SPEC:93;
     * @test_Strategy: In processAction(), Checks that getAuthType() returns null if the request 
     *                 was not authenticated. 
     * @assertion: Returns the name of the authentication scheme used for the 
     *             connection between client and portal, for example, 
     *             BASIC_AUTH or CLIENT_CERT_AUTH, or null if there was no 
     *             authentication. 
     */
    public void GetAuthTypeWithoutProtectionTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetAuthTypeWithoutProtectionTestPortlet");
             
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetCharacterEncodingTest
     * @assertion_ids: PORTLET:JAVADOC:1;
     * @test_Strategy: In processAction(), Checks that getCharacterEncoding() returns the encoding 
     *         set by this client in the request.
     * @assertion:  Returns the name of the character encoding used in the 
     *              body of this request. This method returns null if the 
     *              request does not specify a character encoding. 
     */
    public void GetCharacterEncodingTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetCharacterEncodingTestPortlet");





        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        // set the header to be something like "Content-Type:text/plain; charset=ISO-8859-1"
        setRequestProperty(REQUEST_HEADERS, 
                        "Content-Type:text/plain; charset=" + 
                        CommonConstants.CLIENT_CHARSET);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetContentLengthTest
     * @assertion_ids: PORTLET:JAVADOC:2;
     * @test_Strategy: First request is to a portlet that writes a action URL 
     *                 to the response object. In the second request in the 
     *                 prcoessAction() method the getReader() method is invoked.
     *                 Test passes if getContentLength returns the actual 
     *                 number of chars return by the client in this method.
     * @assertion: Returns the length, in bytes, of the request body which is 
     *             made available by the input stream, or -1 if the length is 
     *             not known. 
     */
    public void GetContentLengthTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetContentLengthTestPortlet");

         /*
          * Sets the GET request to be sent out to the server.
          */


        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_PLAIN);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(firstResponse);  
    }

    /*
     * @testName: GetContentTypeTest
     * @assertion_ids: PORTLET:JAVADOC:3;
     * @test_Strategy: Client writes a string with the request with content type
     *              , the portlet makes sure the type returned by 
     *              PortletRequest.getContentType() api is same as used by 
     *              client in processAction().
     * @assertion: Returns the MIME type of the body of the request, or null 
     *             if the type is not known. 
     */
    public void GetContentTypeTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetContentTypeTestPortlet");


        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_PLAIN);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  



    }

    /*
     * @testName: GetContextPathTest
     * @assertion_ids: PORTLET:JAVADOC:170;
     * @test_Strategy: Test spans through three requests.
     *    Second request is made to get a actionURL, written by first request.
     *    Second request returns the context 
     *    path. Third request is made by having a servlet in the same portlet 
     *    app associated with a 
     *    dir. So the test client would first call the portlet get the context 
     *    path and then call the servlet using the context path plus the 
     *    servlet pattern: /<context-path>/<servlet-mapping> If you get the 
     *    servlet content you pass otherwise you fail. 
     * @assertion: Returns the portion of the request URL that indicates the 
     *             context of the request. The context path always comes first 
     *             in a request URL. The path starts with a slash(/ ) 
     *             character but does not end with a slash character. For 
     *             portlets in the default (root) context, this method returns 
     *             an empty string("" ). The container does not decode this 
     *             string. To encode a URL the encodeURL method must be used. 
     */
    public void GetContextPathTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetContextPathTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Invokes the first request to a portlet that would write the portlet
         * context path in the response
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Invokes the test.
         */
        HttpResponse secondResponse = invoke(firstResponse);  

        /*
        * Extract the portlet context path
        */
        PortletTCKCustomClientTag customTag
            = new PortletTCKCustomClientTag(
                CommonConstants.PATH_TO_SERVLET_TAG);

        String pathToServlet = customTag.extractTag(secondResponse);


        /*
        * New request to a servlet in the same application. Request made using
        * the context path.
        */

        setRequestProperty(REQUEST, pathToServlet);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(secondResponse);  
       

    }

    /*
     * @testName: GetLocaleTest
     * @assertion_ids: PORTLET:JAVADOC:171;
     * @test_Strategy: Client sets the locale that it will accept and calls
     *                 portlet. Portlet verifies that getLocale is not null. 
     * @assertion: Returns the preferred Locale in which the portal will 
     *             accept content. The Locale may be based on the 
     *             Accept-Language header of the client. 
     */
    public void GetLocaleTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetLocaleTestPortlet");
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);
        setRequestProperty(REQUEST_HEADERS, "Accept-Language:en-US" );



        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_HEADERS, "Accept-Language:en-US" );

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetParameterNamesTest
     * @assertion_ids: PORTLET:JAVADOC:176;
     * @test_Strategy: First request to the portlet writes a 
     *            action portletURL with paramaters in it, to the output stream. 
     *            The portlet URL string is extracted and used for second 
     *            request. The portlet uses PortletRequest.getParameterNames() 
     *            to see if the expected parameter names are returned.
     * @assertion: Returns an Enumeration of String objects containing the 
     *             names of the parameters contained in this request. If the 
     *             request has no parameters, the method returns an empty 
     *             Enumeration. Only parameters targeted to the current 
     *             portlet are returned. 
     */
    public void GetParameterNamesTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetParameterNamesTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetParameterTest
     * @assertion_ids: PORTLET:JAVADOC:173;
     * @test_Strategy: First request to the portlet writes a 
     *            action portletURL with a paramater in it, to the output stream. 
     *            The portlet URL string is extracted and used for second 
     *            request. The portlet uses PortletRequest.getParameter() 
     *            to see if the expected parameter is returned.
     * @assertion: Returns the value of a request parameter as a String, or 
     *             null if the parameter does not exist. Request parameters 
     *             are extra information sent with the request. The 
     *             returned parameter   are "x-www-form-urlencoded" 
     *             decoded. Only parameters 
     *             targeted to the current portlet are accessible. This method 
     *             should only be used if the parameter has only one value. If 
     *             the parameter might have more than one value, use . If this 
     *             method is used with a multivalued parameter, the value 
     *             returned is equal to the first value in the array returned 
     *             by getParameterValues. 
     */
    public void GetParameterTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetParameterTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  


    }

    /*
     * @testName: GetParameterValuesTest
     * @assertion_ids: PORTLET:JAVADOC:177;PORTLET:SPEC:69;
     * @test_Strategy: First request to the portlet writes a action portletURL with 
     *                 a parameter having 2 values. The portlet URL string is 
     *                 extracted and used for second request. The portlet uses 
     *                 PortletRequest.getParameterValues() to check values. 
     * @assertion: Returns an array of String objects containing all of the 
     *             values the given request parameter has, or null if the 
     *             parameter does not exist. The returned parameters are 
     *             "x-www-form-urlencoded" decoded. If the parameter has 
     *             a single value, the array has a length of 1. 
     */
    public void GetParameterValuesTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetParameterValuesTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetPortalContextTest
     * @assertion_ids: PORTLET:JAVADOC:179;
     * @test_Strategy: In processAction(), Test passes if a 
     *               non-null value for PortalContext object 
     *                 is obtained when this method is invoked.
     * @assertion: Returns the context of the calling portal. 
     */
    public void GetPortalContextTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetPortalContextTestPortlet");



         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(UNEXPECTED_RESPONSE_MATCH,
            ResultWriter.getFailedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  


        /*
         * Extract the PortletURL string with portletmode set in it from the 
		 * content of HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetPortletModeTest
     * @assertion_ids: PORTLET:JAVADOC:180;
     * @test_Strategy: Make a first request to the portlet and set the portlet 
	 *				   mode into the PortletURL response object and then in the 	 
     *				   second request which invokes the processAction() method 
	 *				   verify that the portlet mode is  correct using 
	 *				   request.getPortletMode()
     * @assertion: Returns the current portlet mode of the portlet. 
     */
    public void GetPortletModeTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetPortletModeTestPortlet");

         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(UNEXPECTED_RESPONSE_MATCH,
            ResultWriter.getFailedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  


        /*
         * Extract the PortletURL string with portletmode set in it from the 
		 * content of HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetPortletSessionTest
     * @assertion_ids: PORTLET:JAVADOC:181;
     * @test_Strategy: In processAction(), Calls getPortletSession() and make sure it is not null. 
     *                 (Rest are checked in PortletSession tests.)
     * @assertion: Returns the current portlet session or, if there is no 
     *             current session, creates one and returns the new session.  
     *             Creating a new portlet session will result in creating a 
     *             new HttpSession on which the portlet session is based on. 
     */
    public void GetPortletSessionTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetPortletSessionTestPortlet");
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetPreferencesTest
     * @assertion_ids: PORTLET:JAVADOC:183;
     * @test_Strategy: Define a preference entity for this portlet. Test passes 
     *                 if a non-null value for PortletPreference is obtained 
     *                 when this method is invoked, in processAction()
     * @assertion: Returns the preferences object associated with the portlet. 
     */
    public void GetPreferencesTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                            getDefaultPortletApp(),
                                            "GetPreferencesTestPortlet");

        
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    
    }

    /*
     * @testName: GetPortletSessionParamTest
     * @assertion_ids: PORTLET:JAVADOC:182;
     * @test_Strategy: In processAction(),  calls the 
	 *				   getPortletSession() with true and makes sure the session 
	 *				   is returned and is not null. Now we save the object id 
	 *				   and call the api again with(false and true). The 
	 *				   session object returned should be the same.
     * @assertion: Returns the current portlet session or, if there is no 
     *             current session and the given flag is true, creates one and 
     *             returns the new session. If the given flag is false and 
     *             there is no current portlet session, this method returns 
     *             null. Creating a new portlet session will result in creating 
     *             a new HttpSession on which the portlet session is based on.
     *             getPortletSession(false) is untestable, since 
     *             implementations might already have a session.
     */
    public void GetPortletSessionParamTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetPortletSessionParamTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetWindowStateTest
     * @assertion_ids: PORTLET:JAVADOC:197;
     * @test_Strategy: Make a first request to the portlet and that writes  
	 *				   a action PortletURL in response stream. In 
	 *				   the second request which invokes the processAction() 
	 *				   method verify that the window state is no NORMAL using 
	 *				   request.getWindowState()
     * @assertion: Returns the current window state of the portlet. 
     */
    public void GetWindowStateTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetWindowStateTestPortlet");

         /*
          * Sets the GET request to be sent out to the server.
          */
        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(UNEXPECTED_RESPONSE_MATCH,
            ResultWriter.getFailedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  


        /*
         * Extract the PortletURL string with portletmode set in it from the 
		 * content of HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

	/*
     * @testName: GetParameterMapTest
     * @assertion_ids: PORTLET:JAVADOC:175;PORTLET:SPEC:71;
     * @test_Strategy: Test for PortletRequest.getParameterMap() method
     *                  First request to the portlet writes a
     *                  action portletURL with few paramaters in it, 
     *                  to the output stream.
     *                  The portlet URL string is extracted and used for second
     *                  request. In the second request, in processAction,
     *                  portlet uses PortletRequest.getParameterMap()
     *                  to see if the expected parameter is returned.
     * @assertion: Returns a Map of the parameters of this request. Request 
     *             parameters are extra information sent with the request.
     */
    public void GetParameterMapTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetParameterMapTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetInvalidParameterTest
     * @assertion_ids: PORTLET:JAVADOC:173;
     * @test_Strategy: A negative test. If the paramater is not defined, 
     *             getParameter() method should return null.
     * @assertion: Returns the value of a request parameter as a String, or 
     *             null if the parameter does not exist. Request parameters 
     *             are extra information sent with the request. The 
     *             returned parameter   are "x-www-form-urlencoded" 
     *             decoded. Only 
     *             parameters targeted to the current portlet are accessible.  
     *             This method should only be used if the parameter has only 
     *             one value. If the parameter might have more than one value, 
     *             use . If this method is used with a multivalued parameter, 
     *             the value returned is equal to the first value in the array 
     *             returned by getParameterValues. 
     */
    public void GetInvalidParameterTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetInvalidParameterTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: GetReaderInProcessActionTest
     * @assertion_ids: PORTLET:JAVADOC:7;
     * @test_Strategy: First request is to a portlet that writes a Action URL 
     *                 to the response object. In the second request in the 
     *                 processAction() method the getReader() method is invoked.
     *                 Test passes if no exception is thrown and the client 
     *                 contents are successfully read when this method is 
     *                 executed.
     * @assertion: Retrieves the body of the HTTP request from the client to 
     *             the portal as character data using a BufferedReader. The 
     *             reader translates the character data according to the 
     *             character encoding used on the body. Either this method or 
     *             getPortletInputStream may be called to read the body, not both. If the client 
     *             request has been triggered by an action URL, the 
     *             BufferedReader is available in the processAction method. If 
     *             the client request has been triggered by a render URL, the 
     *             BufferedReader is available in the render method. This 
     *             method can only be called by the portlet that was the 
     *             target of a user action request. For HTTP POST data of type 
     *             application/x-www-form-urlencoded this method throws an 
     *             IllegalStateException as this data has been already 
     *             processed by the portal/portlet-container and is available 
     *             as request parameters. 
     */
    public void GetReaderInProcessActionTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetReaderInProcessActionTestPortlet");

         /*
          * Sets the GET request to be sent out to the server.
          */


        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_PLAIN);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }


    /*
     * @testName: GetReaderIllegalStateAfterInputStreamTest
     * @assertion_ids: PORTLET:JAVADOC:10;PORTLET:SPEC:98;
     * @test_Strategy: First request is to a portlet that writes a action URL 
     *                 to the response object. In the second request, in the 
     *                 processAction() method the getInputStream() is invoked before 
     *                 getReader() method. 
     *                 Test passes if IllegalStateException exception is thrown.
     * @assertion: if getPortletInputStream method has been called on this 
     *             request, it is a HTTP POST data of type 
     *             application/x-www-form-urlencoded, or is not target of a 
     *             user action request, or the portlet has already written 
     *             something to the output stream.
     */
    public void GetReaderIllegalStateAfterInputStreamTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetReaderIllegalStateAfterInputStreamTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using URL from the first response. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
        String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

        /*
         * Sets the second POST request to be sent out to the server.
         */

        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_MULTI_PART);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(firstResponse);  
    }



     /*
     * @testName: GetReaderIllegalStateWithApplFormPostDataTest
     * @assertion_ids: PORTLET:JAVADOC:10;PORTLET:SPEC:99;
     * @test_Strategy: First request to the portlet writes a 
     *            portletURL to the output stream. 
     *            The portlet URL string is extracted and used for second 
     *            request. Second request has application-form post data.
     *            The portlet calls PortletRequest.getReader() 
     *            to see if an exception is thrown.
     * @assertion: if getPortletInputStream method has been called on this 
     *             request, it is a HTTP POST data of type 
     *             application/x-www-form-urlencoded, or is not target of a 
     *             user action request, or the portlet has already written 
     *             something to the output stream.
     */
    public void GetReaderIllegalStateWithApplFormPostDataTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                            getDefaultPortletApp(),
                            "GetReaderIllegalStateWithApplFormPostDataTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request that has added POST parameters 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
          
        String secondRequest = getPortalReturnURL(portletURLStr);
        
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(REQUEST_HEADERS, 
                "Content-Type:application/x-www-form-urlencoded"); 

        setRequestProperty(CONTENT, 
                CommonConstants.PARAM_NAME 
                + "=" 
                + CommonConstants.PARAM_POST_VALUE);


        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }




    /*
     * @testName: GetInputStreamInProcessActionTest
     * @assertion_ids: PORTLET:JAVADOC:4;
     * @test_Strategy: First request is to a portlet that writes a Action URL 
     *                 to the response object. In the second request in the 
     *                 processAction() method the getInputStream() method is 
     *                 invoked. Test passes if no exception is thrown and the 
     *                 client contents are successfully read when this method 
     *                 is executed.
     * @assertion: Retrieves the body of the HTTP request from client to 
     *             portal as binary data using an InputStream. Either this 
     *             method or getReader may be called to read the body, but not both. If 
     *             the client request has been triggered by an action URL, 
     *             the InputStream is available in the processAction method. 
     *             If the client request has been triggered by a render URL, 
     *             the InputStream is available in the render method. This 
     *             method can only be called by the portlet that was the 
     *             target of a user action request. For HTTP POST data of type 
     *             application/x-www-form-urlencoded this method throws an 
     *             IllegalStateException as this data has been already 
     *             processed by the portal/portlet-container and is available 
     *             as request parameters. 
     */
    public void GetInputStreamInProcessActionTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetInputStreamInProcessActionTestPortlet");

         /*
          * Sets the GET request to be sent out to the server.
          */


        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_PLAIN);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetInputStreamIllegalStateAfterReaderTest
     * @assertion_ids: PORTLET:JAVADOC:6;PORTLET:SPEC:98;
     * @test_Strategy: First request is to a portlet that writes a action URL 
     *                 to the response object. In the second request in the 
     *                 processAction() method the getInputStream() is invoked after 
     *                 getReader() method. 
     *                 Test passes if IllegalStateException exception is thrown.
     * @assertion: If the portlet accesses this function and is not target of a
     *             user action request, is in the wrong lifecycle phase, 
     *             getReader was already called, it is a HTTP POST data of 
     *             type application/x-www-form-urlencoded, or the portlet has 
     *             already written something to the output stream.
     */
    public void GetInputStreamIllegalStateAfterReaderTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetInputStreamIllegalStateAfterReaderTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using URL from the first response. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
        String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

        /*
         * Sets the second POST request to be sent out to the server.
         */

        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(CONTENT, 
                    CommonConstants.CLIENT_CONTENT);
        setRequestProperty(REQUEST_HEADERS, 
                    CommonConstants.CLIENT_CONTENT_TYPE_HEADER_MULTI_PART);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(firstResponse);  
    }


     /*
     * @testName: GetInputStreamIllegalStateWithApplFormPostDataTest
     * @assertion_ids: PORTLET:SPEC:99;PORTLET:JAVADOC:6;
     * @test_Strategy: First request to the portlet writes a 
     *            portletURL to the output stream. 
     *            The portlet URL string is extracted and used for second 
     *            request. Second request has application-form post data.
     *            The portlet calls PortletRequest.getInputStream() 
     *            to see if an exception is thrown.
     * @assertion: If the portlet accesses this function and is not target of a
     *             user action request, is in the wrong lifecycle phase, 
     *             getReader was already called, it is a HTTP POST data of 
     *             type application/x-www-form-urlencoded, or the portlet has 
     *             already written something to the output stream.
     */
    public void GetInputStreamIllegalStateWithApplFormPostDataTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                            getDefaultPortletApp(),
                            "GetInputStreamIllegalStateWithApplFormPostDataTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request that has added POST parameters 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second POST request to be sent out to the server.
          */
          
        String secondRequest = getPortalReturnURL(portletURLStr);
        
        setRequestProperty(REQUEST, secondRequest);
        setRequestProperty(REQUEST_METHOD, "POST");
        setRequestProperty(REQUEST_HEADERS, 
                "Content-Type:application/x-www-form-urlencoded"); 

        setRequestProperty(CONTENT, 
                CommonConstants.PARAM_NAME 
                + "=" 
                + CommonConstants.PARAM_POST_VALUE);


        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }



    /*
     * @testName: GetRemoteUserNullTest
     * @assertion_ids: PORTLET:JAVADOC:189;PORTLET:SPEC:60;
     * @test_Strategy: Negative Testing. Checks that getRemoteUser() returns 
     *                 null if the user has not been authenticated. 
     * @assertion: Returns the login of the user making this request, if the 
     *             user has been authenticated, or null if the user has not 
     *             been authenticated. 
     */
    public void GetRemoteUserNullTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetRemoteUserNullTestPortlet");

        
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetResponseContentTypeTest
     * @assertion_ids: PORTLET:JAVADOC:191;
     * @test_Strategy: Check that return value of 
	 *				   PortletRequest.getResponseContentType() is not null.
     * @assertion: Returns the portal preferred content type for the response.  
     *             The content type only includes the MIME type, not the 
     *             character set. 
     */
    public void GetResponseContentTypeTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetResponseContentTypeTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: GetResponseContentTypesTest
     * @assertion_ids: PORTLET:JAVADOC:192;PORTLET:SPEC:94;
     * @test_Strategy: Checks that return value of this method is not null.
	 *				   And the first item in the list is same as returned by
	 *				   getResponseContentType().
     * @assertion: Gets a list of content types which the portal accepts for 
     *             the response. This list is ordered with the most preferable 
     *             types listed first. The content type only includes the MIME 
     *             type, not the character set. 
     */
    public void GetResponseContentTypesTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "GetResponseContentTypesTestPortlet");



         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(UNEXPECTED_RESPONSE_MATCH,
            ResultWriter.getFailedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  


        /*
         * Extract the PortletURL string with portletmode set in it from the 
		 * content of HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: CheckIsPortletModeAllowedTest
     * @assertion_ids: PORTLET:JAVADOC:198;
     * @test_Strategy: The VIEW portlet mode must be supported , so we'll call 
	 *				   this method with VIEW portlet mode and make sure it 
	 *				   returns true.
     * @assertion: Returns true, if the given portlet mode is a valid one to 
     *             set for this portlet in the context of the current request. 
     *             This method can be used before calling 
     *             ActionResponse.setPortletMode to avoid an exception. 
     */
    public void CheckIsPortletModeAllowedTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "CheckIsPortletModeAllowedTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: IsRequestedSessionIdValidForValidSessionTest
     * @assertion_ids: PORTLET:JAVADOC:199;
     * @test_Strategy: First request issued to a portlet that will just return
     *                 a portlet url string. In the second request the method 
     *                 isRequestedSessionIdValid() is invoked. Test passses if 
     *                 this method returns true.
     * @assertion: Checks whether the requested session ID is still valid. 
     */
    public void IsRequestedSessionIdValidForValidSessionTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                      getDefaultPortletApp(),
                         "IsRequestedSessionIdValidForValidSessionTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: IsRequestedSessionIdValidForInvalidSessionTest
     * @assertion_ids: PORTLET:JAVADOC:199;
     * @test_Strategy: A negative test for 
     *          PortletRequest.isRequestedSessionIdValid() method
     *          First request issued to a portlet just writes a PortletURL 
     *          to output stream. Second request will invalidate
     *          the session. Then the method isRequestedSessionIdValid()
     *          is invoked. Test passses if this method returns false.
     * @assertion: Checks whether the requested session ID is still valid. 
     */
    public void IsRequestedSessionIdValidForInvalidSessionTest() throws Fault {

        TSPortletInfo portletInfo = new TSPortletInfo(
                                      getDefaultPortletApp(),
                     "IsRequestedSessionIdValidForInvalidSessionTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: CheckIsWindowStateAllowedTest
     * @assertion_ids: PORTLET:JAVADOC:202;
     * @test_Strategy: The NORMAL window state must be supported , so we'll 
	 *				   call this method with NORMAL window state and make sure
	 *				   it returns true.
     * @assertion: Returns true, if the given window state is valid to be set 
     *             for this portlet in the context of the current request. This
     *             method should be used before calling 
     *             ActionResponse.setWindowState to avoid an exception.
     */
    public void CheckIsWindowStateAllowedTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "CheckIsWindowStateAllowedTestPortlet");

        
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }

    /*
     * @testName: RemoveAttributeTest
     * @assertion_ids: PORTLET:JAVADOC:203;
     * @test_Strategy: Add an attribute, then remove an attribute and then 
	 *					check that attribute is not there in the request.
     * @assertion: Removes an attribute from this request. This method is not 
     *             generally needed, as attributes only persist as long as the 
     *             request is being handled. Attribute names should follow the 
     *             same conventions as package names. Names beginning with 
     *             java.*, javax.*, and com.sun.* are reserved for use by 
     *             Sun Microsystems. 
     */
    public void RemoveAttributeTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "RemoveAttributeTestPortlet");
        
        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  

    }

    /*
     * @testName: SetAttributeTest
     * @assertion_ids: PORTLET:JAVADOC:205;
     * @test_Strategy: Set an attribute using setAttribute() method and make 
     *                 sure we can get it using getAttribute().
     * @assertion: Stores an attribute in this request. 
     *              Attribute names should follow 
     *             the same conventions as package names. Names beginning with 
     *             java.*, javax.*, and com.sun.* are reserved for use by 
     *             Sun Microsystems. If the value passed into this method is 
     *             null, the effect is the same as calling 
     *             removeAttribute(java.lang.String).  
     */
    public void SetAttributeTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "SetAttributeTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }


    /*
     * @testName: SetAttributeNullTest
     * @assertion_ids: PORTLET:JAVADOC:205;
     * @test_Strategy: Set an attribute using setAttribute() method then
     *                 set it again with null value.
     *                 Check to make sure we sure we don't get this 
     *                 attribute in the getAttributeNames
     * @assertion: Stores an attribute in this request. 
     *              Attribute names should follow 
     *             the same conventions as package names. Names beginning with 
     *             java.*, javax.*, and com.sun.* are reserved for use by 
     *             Sun Microsystems. If the value passed into this method is 
     *             null, the effect is the same as calling 
     *             removeAttribute(java.lang.String).  
     */
    public void SetAttributeNullTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "SetAttributeNullTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }


    /*
     * @testName: SetCharacterEncodingIllegalStateTest
     * @assertion_ids: PORTLET:JAVADOC:13;
     * @test_Strategy: Invoke the setCharacterEncoding() method after invoking 
     *                 the getReader() method. Test passes if an 
     *                 IllegatStateException() is thrown.
     * @assertion: If this method is called after reading request parameters 
     *             or reading input using getReader()
     */
    public void SetCharacterEncodingIllegalStateTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                  getDefaultPortletApp(),
                                 "SetCharacterEncodingIllegalStateTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To get a portlet url string written by the portlet 
         ******************************************************************/
         
         /*
          * Sets the GET request to be sent out to the server.
          */

        String firstRequest = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, firstRequest);


        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  

        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned 
         * in previous request. 
         ******************************************************************/

        /*
         * Extract the PortletURL string from the content of 
         * HttpResponse of the first pass
         */
         String portletURLStr = PortletURLClientTag.extractContent(
                                        firstResponse);

         /*
          * Sets the second GET request to be sent out to the server.
          */
        String secondRequest = getPortalReturnURL(portletURLStr);
        setRequestProperty(REQUEST, secondRequest);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */

        invoke(firstResponse);  
    }
}
