/**
 * Copyright 2007 IBM Corporation.
 */

package com.sun.ts.tests.portlet.api.javax_portlet.PortletFilter;

import com.sun.javatest.Status;
import com.sun.ts.tests.common.webclient.http.HttpResponse;
import com.sun.ts.tests.portlet.common.client.BasePortletUrlClient;
import com.sun.ts.tests.portlet.common.client.TSPortletInfo;
import com.sun.ts.tests.portlet.common.client.tags.PortletURLClientTag;
import com.sun.ts.tests.portlet.common.util.ResultWriter;

/**
 * URLClient and SpecURLClient are used as HTTP test clients to test
 * the Portlet API and specification assertions.  Each assertion test
 * is performed by executing a method in the *URLClient class that has
 * the @testName, @assertion_ids, @test_Strategy, and @assertion tags
 * in its javadoc comments.
 * <p>
 * In each test, at least one HTTP request is made to retrieve the
 * portal page that interacts with portlets that participate in the
 * corresponding test.  These test portlets/servlets, bundled in WAR
 * files, are assumed to be already deployed on the portal server
 * before the test is run.  The initial URL of the portal page is
 * obtained, using either declarative or programmatic configuration,
 * by the base class method <code>getPortalURL</code>, and is set for
 * the test using the <code>setRequestProperty</code> method.  See the
 * Technology Compatibility Kit Requirements chapter of the Portlet
 * Specification for details.  Subsequent requests for the test are
 * done using URLs, generated by PortletURL, that are part of the
 * returned portal pages.
 * <p>
 * The test sets the test success criteria using the
 * <code>setCriteriaProperty</code> method to look for either expected
 * or unexpected substrings in the portal page returned to decide
 * whether a test has passed or failed.
 * <p>
 * Finally, the <code>invoke()</code> method makes the HTTP request
 * and validates the output for the configured success criteria.  In
 * case of failure, this method throws a <code>Fault</code> exception
 * that is caught by the framework to report a failure for the test. 
 */
public class SpecURLClient extends BasePortletUrlClient {
    public static void main(String[] args) {
        SpecURLClient theTests = new SpecURLClient();
        Status status = theTests.run(args, System.out, System.err);
        status.exit();
    }
    /*
	 * @class.setup_props: ts_home;
	 */
    
    /**
     * Returns the name of the default portlet app.
     */
    public String getDefaultPortletApp() {
        return "portlet_jp_PortletFilter_web";
    }

    /*
     * @testName: InitFilterTest
     * @assertion_ids: PORTLET:SPEC:293;
     * @test_Strategy: Checks in doFilter() that init(FilterConfig config) has been called.
     * @assertion: The portlet container must ensure that it has instantiated
     * 			   a filter of the appropriate class for each filter in the list,
     * 			   and called its init(FilterConfig config) method.
     */
    public void InitFilterTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "InitFilterTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke();
    }
    
    /*
     * @testName: InterfaceFilterTest
     * @assertion_ids: PORTLET:SPEC:294;PORTLET:SPEC:300;PORTLET:SPEC:301;
     * @test_Strategy: Checks in doFilter that destroy() has not been called.
     * @assertion: Depending on the target method of doFilter call the
     * 			   PortletRequest and PortletResponse must be instances
     * 			   of the following interfaces.
     */
    public void InterfaceFilterTest() throws Fault {
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "InterfaceFilterTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To send a request using portletinfo. Test Render
         * 				Filter Interface
         ******************************************************************/
        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  
        /*******************************************************************
         * SECOND TRIP:: To send a request using PortletURL returned in 
         *  previous request. Action
         ******************************************************************/

        String secondRequest = PortletURLClientTag.extractContent(
        		firstResponse);
        setRequestProperty(REQUEST, secondRequest);
        
        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        HttpResponse secondResponse = invoke(firstResponse);
        /*******************************************************************
         * THIRD TRIP:: To send a request using PortletURL returned in 
         *  previous request. ResourceServing
         ******************************************************************/
        String thirdRequest = PortletURLClientTag.extractContent(
        		secondResponse);
        setRequestProperty(REQUEST, thirdRequest);
        
        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(secondResponse);
    }

    /*
     * @testName: InterfaceFilterEventTest
     * @assertion_ids: PORTLET:SPEC:294;PORTLET:SPEC:300;PORTLET:SPEC:301;
     * @test_Strategy: Checks in doFilter that destroy() has not been called.
     * @assertion: Depending on the target method of doFilter call the
     * 			   PortletRequest and PortletResponse must be instances
     * 			   of the following interfaces.
     */
    public void InterfaceFilterEventTest() throws Fault{
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "InterfaceFilterEventTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To send a request for action-URL.
         ******************************************************************/
        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);
        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();  
        /*******************************************************************
         * SECOND TRIP:: To send a request using portletinfo. Test Event
         * 				Filter Interface
         ******************************************************************/

        String portletURLStr = PortletURLClientTag.extractContent(
           		firstResponse);
           
        String secondRequest = getPortalReturnURL(portletURLStr);
        
        setRequestProperty(REQUEST, secondRequest);
        
        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */
        invoke(firstResponse);
    }
    /*
     * @testName: DestroyFilterTest
     * @assertion_ids: PORTLET:SPEC:295;
     * @test_Strategy: Checks in doFilter that destroy() has not been called.
     * @assertion: Before a filter instance can be removed from service by
     * 			   the portlet container, the portlet container must first
     * 			   call the destroy method on the filter to enable the
     * 			   filter to release any resources and perform other
     * 			   cleanup operations.
     */
    public void DestroyFilterTest() throws Fault {
        /*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "DestroyFilterTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */invoke();
    }

    /*
     * @testName: WrapperTest
     * @assertion_ids: PORTLET:SPEC:296;
     * @test_Strategy: Checks in doFilter that destroy() has not been called.
     * @assertion: Before a filter instance can be removed from service by
     * 			   the portlet container, the portlet container must first
     * 			   call the destroy method on the filter to enable the
     * 			   filter to release any resources and perform other
     * 			   cleanup operations.
     */
    public void WrapperTest() throws Fault{
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "WrapperTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */invoke();
    }
    /*
     * @testName: OneFilterObjectPerJVMTest
     * @assertion_ids: PORTLET:SPEC:297;
     * @test_Strategy: It tests that there is only one Filter object per
     * 				   Filter definition per JVM.
     * @assertion: The portlet container must instantiate exactly one
     * 			   instance of the Java class defining the filter
     *             per filter declaration in the deployment descriptor.
     */
    public void OneFilterObjectPerJVMTest() throws Fault{
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "OneFilterObjectPerJVMTestPortlet");

        /*******************************************************************
         * FIRST TRIP:: To send a request for run filter first time.
         ******************************************************************/
        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        

        /*
         * Invokes the test.
         */
        HttpResponse firstResponse = invoke();
        
        /*******************************************************************
         * SECOND TRIP:: To send a request for run filter first time.
         ******************************************************************/
        
        String secondrequest = PortletURLClientTag.extractContent(firstResponse);
         setRequestProperty(REQUEST, secondrequest);

         /*
          * Sets the test criteria.
          */
         setCriteriaProperty(SEARCH_STRING,
             ResultWriter.getPassedString(getTestName()));

         /*
          * Invokes the test.
          */invoke(firstResponse);
    }
    /*
     * @testName: WildcardTest
     * @assertion_ids: PORTLET:SPEC:298;
     * @test_Strategy: Tests if the PortletFilter is been associated with
     * 				   the filter.
     * @assertion: Filters can be associated with groups of portlets using
     * 			   the * character as a wildcard at the end of a string
     * 			   to indicate that the filter must be applied to any
     * 			   portlet whose name starts with the characters before
     * 			   the * character.
     */
    public void WildcardTest() throws Fault{
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "WildcardTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */invoke();
    }
    
    /*
     * @testName: FilterNotRemoveTest
     * @assertion_ids: PORTLET:SPEC:299;
     * @test_Strategy: Invokes a portlet and tests if all filters for the
     * 				   portlets are called.
     * @assertion: The portlet container is free to add additional filters
     * 			   at any place in this filter chain, but must not remove
     * 			   filters matching a specific portlet.
     */
    public void FilterNotRemoveTest() throws Fault{
    	/*
         * Sets the GET request to be sent out to the server.
         */
        TSPortletInfo portletInfo = new TSPortletInfo(
                                        getDefaultPortletApp(),
                                        "FilterNotRemoveTestPortlet");

        String request = getPortalURL(portletInfo);
        setRequestProperty(REQUEST, request);

        /*
         * Sets the test criteria.
         */
        setCriteriaProperty(SEARCH_STRING,
            ResultWriter.getPassedString(getTestName()));

        /*
         * Invokes the test.
         */invoke();
    }
    

}
